var gameThrottle = []
var drivingMode = 'Economy+';

// converts some data to other units
function Conversion() {
    this.psiToBar = function(psi) {
        return psi * 0.0689476
    }

    this.barToPsi = function(bar) {
        return bar * 14.5038
    }

    this.kmhToMph = function(kmh) {
        return kmh * 0.621371
    }

    this.mphToKmh = function(mph) {
        return mph * 1.60934
    }

    this.literToGallons = function(liter) {
        return liter * 0.264172
    }
}

function getCookie(name) {
    var name = name + "=";
    var decodedCookie = decodeURIComponent(document.cookie);
    var ca = decodedCookie.split(';');
    for(var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ') c = c.substring(1);
        if (c.indexOf(name) == 0) {
            c = c.substring(name.length, c.length)
            if (Boolean(c)) c = c == 'true'
            return c;
        }
    }
    return false;
}

function setCookie(name, value) {
    document.cookie = `${name}=${value}; expires=Fri, 31 Dec 9999 23:59:59 GMT`;
}

function cookieExists(name) {
    var cookies = decodeURIComponent(document.cookie).split(';');
    for (var i = 0; i <  cookies.length; i++) {
        if (cookies[i].trim().split('=')[0] === name) return true
    }
    return false
}

// change truck telemetry data
function Truck(truck, utils, conversion) {
    this.speed = function () {
        if (showAll) truck.speed = 100
        speed = Math.abs(Math.round(truck.speed) > 0 ? truck.speed : 0)
        return options.mph ? conversion.kmhToMph(speed) : speed;
    }
    
    this.roundedSpeed = function() {
        return utils.formatFloat(truck.speed);
    }

    this.cruiseControlOn = function() {
        if (showAll) truck.cruiseControlOn = true
        return truck.cruiseControlOn
    }

    this.cruiseControlSpeed = function() {
        if (showAll) truck.cruiseControlSpeed = 80


        var cruiseControlSpeed = options.mph 
            ? conversion.kmhToMph(truck.cruiseControlSpeed) 
            : truck.cruiseControlSpeed;

        // console.log(cruiseControlSpeed, truck.cruiseControlSpeed)
        return truck.cruiseControlOn
            ? cruiseControlSpeed
            : 0;
    }

    this.cruiseControlSpeedRounded = function() {
        return truck.cruiseControlOn
            ? parseInt(truck.cruiseControlSpeed)
            : 0;
    }

    this.engineRpm = function() {
        if (showAll) truck.engineRpm = 1300
        return truck.engineRpm / 100
    }

    this.lightsBeamHighOn = function() {
        return showAll
            ? true
            : truck.lightsBeamHighOn
                && truck.lightsBeamLowOn
                && truck.lightsParkingOn;
    }

    this.airPressurePsi = function() {
        return truck.airPressure * 0.95
    }

    this.airPressureBar = function() {
        return conversion.psiToBar(truck.airPressure)
    }

    this.airPressurePsiText = function() {
        return truck.airPressurePsi.toFixed(1)
    }

    this.airPressureBarText = function() {
        return truck.airPressureBar.toFixed(1)
    }

    this.airPressurePercentage = function() {
        var bar = (100 / 14) * (truck.airPressureBar > 14
            ? 14
            : truck.airPressureBar)

        var psi = (100 / 150) * (truck.airPressurePsi > 150
                ? 150
                : truck.airPressurePsi)

        return options.mph
            ? psi
            : bar
    }

    this.airPressureText = function() {
        return options.mph ? truck.airPressurePsiText : truck.airPressureBarText
    }

    this.airPressureUnit = function() {
        return options.mph ? 'psi' : 'bar'
    }

    this.odometerFromStart = function() {
        options.data.odometerFromStart = !options.data.odometerFromStart 
                ? truck.odometer
                : options.data.odometerFromStart;
        return options.data.odometerFromStart
    }

    this.fuelFromStart = function() {
        options.data.fuelFromStart = !parseInt(truck.speedRounded)
                ? truck.fuel
                : options.data.fuelFromStart;
        return options.data.fuelFromStart
    }

    this.distanceFromStart = function() {
        return utils.formatFloat(truck.odometer - truck.odometerFromStart, 3)
    }

    this.fuelAverageConsumption = function() {
        var avg = Math.floor(parseFloat(utils.formatFloat(truck.fuelAverageConsumption * 1000, 1))) / 10
        return options.mph
            ? conversion.literToGallons(conversion.mphToKmh(avg))
            : avg
    }

    this.fuelAverageConsumptionPercentage = function(max = 400) {
        var consumption = truck.fuelAverageConsumption > 100
            ? 100
            : truck.fuelAverageConsumption
        return (max / 100) * consumption;

    }

    this.fuelPercentage = function() {
        return (100 / truck.fuelCapacity) * truck.fuel
    }

    this.adbluePercentage = function() {
        return (100 / truck.adblueCapacity) * truck.adblue
    }

    this.maxGears = function() {
        return showAll
            ? 12
            : truck.displayedGear === 0
                ? 0
                : truck.displayedGear > 0
                    ? Math.floor(truck.forwardGears)
                    : Math.floor(truck.reverseGears)
    }

    this.displayedGear = function() {
        return showAll
            ? 'A'
            : truck.displayedGear === 0
                ? 'N' : (truck.displayedGear > 0
                    ? (['arcade', 'automatic'].includes(truck.shifterType) ? 'A' : 'G')
                    : 'R');
    }

    this.drivingMode = function(max = 200) {
        var avg = drivingMode
        if (['arcade', 'automatic'].includes(truck.shifterType)) {
            if (gameThrottle.length === max) {
                truck.displayedGear !== 'R'
                    ? (drivingMode = ((gameThrottle.reduce((a, b) => a + b) / max) > 0.7)
                        ? 'Sport+'
                        : 'Economy+')
                    : 'everse'
                gameThrottle = []
            }
            gameThrottle.push(truck.gameThrottle)
        } else {
            avg = truck.displayedGear === 'N'
                ? ''
                : truck.displayedGear === 'R'
                    ? 'Reverse'
                    : 'Forward'
        }
        return showAll ? 'Economy+' : avg
    }

    this.gear = function() {
        return showAll
            ? 8
            : truck.gear ? Math.floor(Math.abs(truck.gear)): ''
    }

    this.wearEngine = function() {
        if (showAll) truck.wearEngine = 1
        return (truck.wearEngine * 100) * 0.3619909502262443
    }

    this.wearTransmission = function() {
        if (showAll) truck.wearTransmission = 1
        return (truck.wearTransmission * 100) * 0.3619909502262443
    }
    
    this.wearCabin = function() {
        if (showAll) truck.wearCabin = 1
        return (truck.wearCabin * 100) * 0.3619909502262443
    }
    
    this.wearChassis = function() {
        if (showAll) truck.wearChassis = 1
        return (truck.wearChassis * 100) * 0.3619909502262443
    }
    
    this.wearWheels = function() {
        if (showAll) truck.wearWheels = 1
        return (truck.wearWheels * 100) * 0.3619909502262443
    }

    this.engineHasDamage = function() {
        return Math.floor(truck.wearEngine) ? true : false
    }

    this.transmissionHasDamage = function() {
        return Math.floor(truck.wearTransmission) ? true : false
    }

    this.cabinHasDamage = function() {
        return Math.floor(truck.wearCabin) ? true : false
    }

    this.chassisHasDamage = function() {
        return Math.floor(truck.wearChassis) ? true : false
    }

    this.wheelsHasDamage = function() {
        return Math.floor(truck.wearWheels) ? true : false
    }

    this.wear = function() {
        var wear = Math.floor(
            truck.wearEngine +
            truck.wearTransmission +
            truck.wearCabin +
            truck.wearChassis +
            truck.wearWheels
        )
        
        return wear > 100 ? 100 : wear;
    }

    this.motorControllLightOn = function() {
        return showAll
            ? true
            : ((truck.wearEngine + truck.wearTransmission) / 2) > 10 
                ? true 
                : false;
    }

    this.retarderOn = function() {
        if (showAll) truck.retarderBrake = 1
        return truck.retarderBrake > 0
    }

    this.motorBrakeOn = function() {
        return showAll ? true : truck.motorBrakeOn
    }

    this.blinkerLeftOn = function() {
        return showAll ? true : truck.blinkerLeftOn
    }

    this.blinkerRightOn = function() {
        return showAll ? true : truck.blinkerRightOn
    }

    this.lightsParkingOn = function() {
        return showAll ? true : truck.lightsParkingOn
    }

    this.lightsBeamLowOn = function() {
        return showAll ? true : truck.lightsBeamLowOn
    }

    this.lightsBeaconOn = function() {
        return showAll ? true : truck.lightsBeaconOn
    }

    this.mph = function() {
        return options.mph
    }

    this.mphValue = function() {
        return truck.mph ? 'true' : 'false'
    }

    this.fuelAverageConsumptionUnit = function() {
        return options.mph ? 'g / 100 mi' : 'l / 100 km'
    }

    this.fuelAverageFromStartText = function() {
        return I18N.fromStart()
    }

    this.odometer = function() {
        var odometer = showAll ? 3263527 : truck.odometer;
        return options.mph
            ? conversion.kmhToMph(odometer)
            : odometer;
    }
}

// change trailer telemetry data
function Trailer(trailer, utils, conversion) {
    this.mass = function() {
        if (showAll) trailer.mass = 17300;
        return trailer.attached && trailer.mass
            ? utils.formatFloat(trailer.mass / 1000, 1) + 't'
            : '';
    }

    this.attached = function() {
        return showAll ? true : trailer.attached
    }
}

// change game telemetry data
function Game(game, utils, conversion) {
    this.day = function() {
        return game.connected
            ? Funbit.Ets.Telemetry.Strings.dayOfTheWeek[(new Date(game.time)).getUTCDay()]
            : ''
    }

    this.time = function() {
        if (game.connected) {
            var date = new Date(game.time)
            var hours = date.getHours().toString().padStart(2, '0'); 
            var minutes = date.getMinutes().toString().padStart(2, '0'); 
        }
        return game.connected ? `${hours}:${minutes}`: '';
    }

    this.nextRestStopTimePercentage = function() {
        var dateTime = new Date(game.nextRestStopTime);
        var hours = dateTime.getUTCHours();
        var minutes = dateTime.getUTCMinutes() + (hours * 60);
        return showAll
            ? 45
            : game.connected
                ? 100 - (Math.round(100 / (11 * 60) * minutes))
                : 0;
    }

    this.connectedValue = function() {
        return game.connected ? 'true' : 'false'
    }

    this.velocityUnit =  function() {
        return options.mph ? 'mph' : 'km/h'
    }

    this.developmentSettings =  function() {
        return showDevelopmentVariables
    }
}

// change navigation telemetry data
function Navigation(navigation, utils, conversion) {
    this.speedLimit = function() {
        if (showAll) navigation.speedLimit = 80
        return options.mph 
            ? Math.round(conversion.kmhToMph(navigation.speedLimit)) 
            : navigation.speedLimit;
    }

    this.estimatedDistance = function() {
        var distance = showAll 
            ? 3146
            : navigation.estimatedDistance / 1000

        return options.mph 
            ? Math.round(conversion.kmhToMph(distance))
            : Math.round(distance)
    }

    this.estimatedDistanceText = function() {
        return navigation.estimatedDistance
            ? `${navigation.estimatedDistance} ${options.mph ? 'mi' : 'km'}`
            : ''
    }
}

// change job telemetry data
function Job(job, utils, conversion) {
    this.cargo = function() {
        return job.income ? true : false;
    }

    this.destinationCity = function() {
        return showAll ? 'Salzbug' : job.destinationCity
    }
}

// Telemetry Manager
function TelemetryManager(data, utils) {
    this.conversion = new Conversion();
    this.game = new Game(data.game, utils, this.conversion);
    this.truck = new Truck(data.truck, utils, this.conversion);
    this.trailer = new Trailer(data.trailer, utils, this.conversion);
    this.navigation = new Navigation(data.navigation, utils, this.conversion);
    this.job = new Job(data.job, utils, this.conversion);

    this.setLang = function() {
        options.lang = (function() {
            return options.hasOwnProperty('lang')
                ? (options.lang).toLowerCase()
                : (navigator.language || navigator.userLanguage).toLowerCase()
        })()
        return options.lang
    }

    // set weekdays
    this.setWeekDays = function() {
        Funbit.Ets.Telemetry.Strings.dayOfTheWeek = (function() {
            var date = new Date();
            Funbit.Ets.Telemetry.Strings.dayOfTheWeek.forEach(function(e, i, a) {
                date = new Date(date.setDate(date.getDate() - date.getDay() + i));
                a[i] = (date.toLocaleDateString(options.lang, {weekday: 'long'}))
            })
            return Funbit.Ets.Telemetry.Strings.dayOfTheWeek
        })();
    }

    this.predefineData = function() {

        this.setLang();
        this.setWeekDays();

        data.game.day = this.game.day();
        data.game.time = this.game.time();
        data.game.nextRestStopTimePercentage = this.game.nextRestStopTimePercentage();
        data.game.connectedValue = this.game.connectedValue();
        data.game.velocityUnit = this.game.velocityUnit();
        data.game.developmentSettings =  this.game.developmentSettings();
    
        data.truck.speed = this.truck.speed();
        data.truck.speedRounded = this.truck.roundedSpeed();
        data.truck.cruiseControlOn = this.truck.cruiseControlOn();
        data.truck.cruiseControlSpeed = this.truck.cruiseControlSpeed();
        data.truck.cruiseControlSpeedRounded = this.truck.cruiseControlSpeedRounded();
        data.truck.engineRpm = this.truck.engineRpm();;
        data.truck.lightsBeamHighOn = this.truck.lightsBeamHighOn();
        data.truck.airPressurePsi = this.truck.airPressurePsi();
        data.truck.airPressureBar = this.truck.airPressureBar();
        data.truck.airPressurePsiText = this.truck.airPressurePsiText();
        data.truck.airPressureBarText = this.truck.airPressureBarText();
        data.truck.airPressurePercentage = this.truck.airPressurePercentage();
        data.truck.odometerFromStart = this.truck.odometerFromStart();
        data.truck.distanceFromStart = this.truck.distanceFromStart();
        data.truck.fuelFromStart = this.truck.fuelFromStart();
        data.truck.fuelAverageConsumption = this.truck.fuelAverageConsumption();
        data.truck.fuelAverageConsumptionPercentage = this.truck.fuelAverageConsumptionPercentage();
        data.truck.fuelPercentage = this.truck.fuelPercentage();
        data.truck.adbluePercentage = this.truck.fuelPercentage();
        data.truck.maxGears = this.truck.maxGears();
        data.truck.displayedGear = this.truck.displayedGear();
        data.truck.drivingMode = this.truck.drivingMode();
        data.truck.gear = this.truck.gear();
        data.truck.wearEngine = this.truck.wearEngine();
        data.truck.wearTransmission = this.truck.wearTransmission();
        data.truck.wearCabin = this.truck.wearCabin();
        data.truck.wearChassis = this.truck.wearChassis();
        data.truck.wearWheels = this.truck.wearWheels();
        data.truck.engineHasDamage = this.truck.engineHasDamage();
        data.truck.transmissionHasDamage = this.truck.transmissionHasDamage();
        data.truck.cabinHasDamage = this.truck.cabinHasDamage();
        data.truck.chassisHasDamage = this.truck.chassisHasDamage();
        data.truck.wheelsHasDamage = this.truck.wheelsHasDamage();
        data.truck.wear = this.truck.wear();
        data.truck.motorControllLightOn = this.truck.motorControllLightOn();
        data.truck.retarderOn = this.truck.retarderOn();
        data.truck.motorBrakeOn = this.truck.motorBrakeOn();
        data.truck.blinkerLeftOn = this.truck.blinkerLeftOn();
        data.truck.blinkerRightOn = this.truck.blinkerRightOn();
        data.truck.lightsParkingOn = this.truck.lightsParkingOn();
        data.truck.lightsBeamLowOn = this.truck.lightsBeamLowOn();
        data.truck.lightsBeaconOn = this.truck.lightsBeaconOn();
        data.truck.mph = this.truck.mph();
        data.truck.mphValue = this.truck.mphValue();
        data.truck.airPressureText = this.truck.airPressureText();
        data.truck.airPressureUnit = this.truck.airPressureUnit();
        data.truck.fuelAverageConsumptionUnit = this.truck.fuelAverageConsumptionUnit();
        data.truck.fuelAverageFromStartText = this.truck.fuelAverageFromStartText()
        data.truck.odometer = this.truck.odometer();

        data.trailer.attached = this.trailer.attached();
        data.trailer.mass = this.trailer.mass();

        data.navigation.speedLimit = this.navigation.speedLimit();
        data.navigation.estimatedDistance = this.navigation.estimatedDistance();
        data.navigation.estimatedDistanceText = this.navigation.estimatedDistanceText();

        data.job.cargo = this.job.cargo();
        data.job.destinationCity = this.job.destinationCity();

        // data['keys'] = this.keys.setKeys(keys);
    }
}

function Utilities() {
    this.drawBackgroundPoints = function(dia = 700, start = 141.9, rounds = 20, hspace = 30, vspace = 3) {
        let end = 180 - start - vspace;
        let difference = (end < start) ? (end += 360) - start: end - start;
        let sections  = Math.floor(difference / vspace)
    
        let svg = document.createElementNS("http://www.w3.org/2000/svg", "svg");
        svg.setAttribute('width', '1920');
        svg.setAttribute('height', '1080');
        
        for (let round = 0; round < rounds; round++) {
            let color = .7 - (round * (.6 / rounds) + .4)
            for (let i = 0; i < sections + 1; i++) {
                let r = (dia / 2) + (hspace * round) + hspace;
                let deg = ((end-start) / hspace) + start + (i * vspace) - vspace;
                let rad = (deg * Math.PI) / 180
                let x = Math.round(r * Math.cos(rad)) + 960
                let y = Math.round(r * Math.sin(rad)) + 545
        
                const circle = document.createElementNS("http://www.w3.org/2000/svg", "circle");
                circle.setAttribute('cx', x);
                circle.setAttribute('cy', y);
                circle.setAttribute('r', '1.5');
                circle.setAttribute('fill', `rgba(255, 255, 255, ${color})`);
        
                svg.appendChild(circle)
            }
        }
    
        $('.points').append(svg)
    }

    this.init = function(data, colors) {
        if (initAnimation) {
            if (data.game.connected) {
                $('.disconnected').fadeOut(300, () => $('.connected').fadeIn(300));
            } else {
                $('.connected').fadeOut(300, () => $('.disconnected').fadeIn(300));
            }
        
            if (!animate) {
                if (data.truck.electricOn) {
                    if (!initialized) {
                        animate = true;
                        $('.engine-off').fadeOut(300, () => {
                            $('.engine-on').show();
                            $('.dashboard').css('backgroundColor', colors.initialize)
                            $('.initialize').fadeIn(300, (e) => {
                                $('.initialize').delay(2000).fadeOut(300, () => {
                                    initialized = true;
                                    $('.dashboard').css('backgroundColor', colors.cockpit)
                                    $('.virtual-cockpit').fadeIn(300, () => {
                                        $('.truck-speed').addClass('load').delay(600).queue(function(next) {
                                            $(this).removeClass('load');
                                            $(this).addClass('fast');
                                            next();
                                        });
                                        $('.truck-engineRpm').addClass('load').delay(600).queue(function(next) {
                                            $(this).removeClass('load');
                                            $(this).addClass('fast');
                                            next();
                                        });
                                        animate = false
                                    });
                                })
                            })
                        })
                    }
                } else {
                    animate = true
                    $('.virtual-cockpit').fadeOut(300, () => {
                        $('.engine-on').hide()
                        $('.dashboard').css('backgroundColor', colors["engine-off"])
                        $('.engine-off').fadeIn(300, () => {
                            animate = false
                            initialized = false
                        })
                    })
                }
            }
        } else {
            $('.connected').show();
            $('.engine-on').show();
            $('.virtual-cockpit').show();
            $('.dashboard').css('backgroundColor', colors.cockpit)
        }
    }

    this.setSpeedLimit = function(limit, start, end, max, dia) {
        let r = dia / 2;
        let deg = (((end-start) / max) * limit) + start;
        let rad = (deg * Math.PI) / 180
        let x = Math.round(r * Math.cos(rad)) + r
        let y = Math.round(r * Math.sin(rad)) + r
    
        const getPoly = () => {
            result = []
            if (limit <= 0 || limit >= max) {
                $('.velocity-warning').css('opacity', 0);
            } else {
                $('.velocity-warning').css('opacity', 1);
                if (x <= 325 && y >= 325) {
                    result.push(`50% 50%, 0% 50%, 0% 100%, ${x}px ${y}px`)
                }
        
                if (x < 325) {
                    result.push((x <= 325 && y <= 325)
                        ? `50% 50%, 50% 0%, 0% 0%, ${x}px ${y}px`
                        : '50% 50%, 50% 0%, 0% 0%, 0% 50%')
                }
        
                result.push((x >= 325 && y <= 325)
                    ? `50% 50%, 497px 49px, 497px 0%, ${x}px ${y}px`
                    : '50% 50%, 50% 0%, 100% 0%, 497px 49px')
            }
    
            return result;
        }
        $('.velocity-warning').css('clipPath', `polygon(${getPoly()})`);
    }

    this.showSpeeding = function(data) {
        let speed = Math.abs(data.truck.speed > 0 ? data.truck.speed : 0);
        if (speed > data.navigation.speedLimit) {
            $('.velocity-warning').css('background', 'radial-gradient(closest-side circle,#000 0,#000 75%, rgb(100, 0, 0) 75%,#000 100%)')
        } else {
            $('.velocity-warning').css('background', 'radial-gradient(closest-side circle,#000 0,#000 75%, rgb(30, 30, 30) 75%,#000 100%)')
        }
    }

    this.gitApiUrl = function(projectId) {
        return `https://gitlab.com/api/v4/projects/${projectId}/repository/branches/master`
    }

    this.checkUpdate = function(projectId) {
        if (!update.checked) {
            $.getJSON( `https://gitlab.com/api/v4/projects/${projectId}/repository/branches/master`, function(master) {
                var skinName = (new URLSearchParams(window.location.search)).get('skin');
                $.get(`skins/${skinName}/.git/refs/heads/master`, function(local) {
                    var local = local.substr(0, local.indexOf('\n'))
                    var remote = master.commit.id
                    if (local !== remote) {
                        update.available = true
                        console.log('update available')
                    }
                    update.checked = true
                }, 'text');
            });
        }
    }

    this.toggleLights = function(activator, target) {
        $(`.${target.join(', .')}`)[activator ? 'addClass' : 'removeClass']('yes');
    }

    function find(txt, start, end) {
        var start = txt.indexOf(start)
        var length = txt.substr(start).indexOf(end)
        return txt.substr(start, length)
    }

    this.mapKeys = function(config) {
        // localStorage.clear()
        
        if (localStorage.getItem('keys') === null) {
            
            var gameKeys = {
                ets2: {
                    keyboard: [],
                    gamepad: []
                },
                ats: {
                    keyboard: [],
                    gamepad: []
                }
            };
            $.each(config.games, function(game, name) {
                $.get(`skins/${config.name}/controls/${game}/controls.sii`).done(function(data) {
                    var lines = data.split("\n");
                    for (var i = 0; i < lines.length; i++) {
                        if (lines[i].indexOf('config_lines[') >= 0) {
                            var gameFunction = ''
                            if (lines[i].indexOf('input') >= 0) {
                                gameFunction = find(lines[i], 'input ', ' `').replace('input', '').trim();
                            }
                            if (lines[i].indexOf('mix') >= 0) {
                                gameFunction = find(lines[i], 'mix ', ' `').replace('mix', '').trim();
                            }
                            if (gameFunction) {
                                var keyArray = find(lines[i], '`', '"').replace(/`/g, '').trim()
                                if (keyArray) {
                                    var keys = keyArray.split('|')
                                    $.each(keys, function(i, e) {
                                        if (e.indexOf('joy') >= 0 || e.indexOf('keyboard') >= 0) {
                                            e = e.replace('?0', '').trim()
                                            if (e.indexOf('keyboard') >= 0) {
                                                if (e.indexOf('modifier') >= 0) {
                                                    if (e.indexOf('no_modifier') >= 0) {
                                                        e = find(e, ',' ,')').replace(',', '').trim();
                                                    } else {
                                                        e = e.substr(e.indexOf('(')).replace(/[()]/g, '').trim().split(',');
                                                        $.each(e, function(i, k, a) {
                                                            e[i] = k.replace('_only', '').replace('keyboard.', '').trim()
                                                        })
                                                    }
                                                } else {
                                                    e = e.replace('keyboard.', '')
                                                }
    
                                                if (gameKeys[game].keyboard.hasOwnProperty(gameFunction)) {
                                                    gameKeys[game].keyboard[gameFunction].push(e)
                                                } else {
                                                    gameKeys[game].keyboard[gameFunction] = []
                                                    gameKeys[game].keyboard[gameFunction].push(e)
                                                }
                                            }
                                            if (e.indexOf('joy') >= 0) {
                                                e = e.replace('ui_', '').replace('joy.', '').trim();
    
                                                if (gameKeys[game].gamepad.hasOwnProperty(gameFunction)) {
                                                    gameKeys[game].gamepad[gameFunction].push(e)
                                                } else {
                                                    gameKeys[game].gamepad[gameFunction] = []
                                                    gameKeys[game].gamepad[gameFunction].push(e)
                                                }
                                            }
                                        }
                                    })
                                }
                            }
                        }
                    }
                    var ets2keys = JSON.stringify(gameKeys.ets2)
                    var atskeys = JSON.stringify(gameKeys.ets2)
                    localStorage.setItem('ets2keys', JSON.stringify(ets2keys))
                }).fail(function() {
                    console.log(`No key assignments found for ${name}`)
                })
            })
        } 
    }

    this.setVelocityUnit = function(data) {
        if (!data.game.connected) {
            options.mph = false
            options.mphPredefined = false
        }
        if (data.game.hasOwnProperty('gameName')) {
            var game = data.game.gameName.toLowerCase()
            if (data.game.connected && !options.mphPredefined) {
                options.game = game;
                if (cookieExists(`${game}Mph`)) {
                    options.mph = getCookie(`${game}Mph`);
                } else {
                    options.mph = game === 'ets2' ? false : true;
                    setCookie(`${game}Mph`, options.mph);
                }
                options.mphPredefined = true
            }
        }
    }
}

function DisplayHelper(data, utils, conversion) {
    this.showPerfectRpm = function(target, styleClass, min, max) {
        if (data.truck.engineRpm >= min && data.truck.engineRpm <= max) {
            $(`.${target}`).addClass(styleClass);
        }
        else {
            $(`.${target}`).removeClass(styleClass);
        }
    }

    this.fuelText = function(types) {
        var litre = options.mph ? I18N.gallons() : I18N.litre()
        $.each(types, function(i, type) {
            var fuel = utils.formatFloat(data.truck[type], 1)
            var capacity = data.truck[`${type}Capacity`]
            if (options.mph) {
                fuel = Math.round(conversion.literToGallons(fuel))
                capacity = Math.round(conversion.literToGallons(capacity))
            }
            $(`.${type}-current-text`).html(`${fuel} ${litre} / ${capacity} ${litre}`)
        })
    }

    this.enterFullscreen = function(element) {
        if(element.requestFullscreen) {
            element.requestFullscreen();
        } else if(element.mozRequestFullScreen) {
            element.mozRequestFullScreen();
        } else if(element.msRequestFullscreen) {
            element.msRequestFullscreen();
        } else if(element.webkitRequestFullscreen) {
            element.webkitRequestFullscreen();
        }
    }

    this.exitFullscreen = function() {
        if(document.exitFullscreen) {
          document.exitFullscreen();
        } else if(document.mozCancelFullScreen) {
          document.mozCancelFullScreen();
        } else if(document.webkitExitFullscreen) {
          document.webkitExitFullscreen();
        }
    }

    this.isFullscreen = function() {
        return (document.exitFullscreen || document.mozCancelFullScreen || document.webkitExitFullscreen)
            ? true
            : false;
    }

    this.fatique = function(target, defaultColor, fatiqueColor, warningColor) {
        var rest = data.game.nextRestStopTimePercentage;
        $(`.${target} > svg > defs > linearGradient > stop.default`).attr('stop-color', defaultColor)
        $(`.${target} > svg > defs > linearGradient > stop.fatique`).attr('stop-color', rest > 70 ? warningColor: fatiqueColor)
        $(`.${target} > svg > defs > linearGradient > stop.percentage`).attr('offset', `${rest}%`)
    }
}

function Translator() {

    this.currency = function() {
        return this.getData({
            'de-de': '€',
            'en-us': '$',
            'bg': 'лв',
            'de-ch': 'CHF',
            'de-fr': 'CHF',
            'cs': 'Kč',
            'da': 'kr',
            'hu': 'ft',
            'pl': 'zł',
            'ru': '₽',
            'tr': '₺',
            'uk': '₴',
            'en-gb': '£'
        })
    }

    this.motorDamage = function() {
        return this.getData({
            'de-de': 'Motorschaden',
            'en-us': 'Motor damage',
        })
    }

    this.fromStart = function() {
        return this.getData({
            'de-de': 'Ab start',
            'en-us': 'From start',
        })
    }

    this.litre = function() {
        return this.getData({
            'de-de': 'liter',
            'en-us': 'litre',
        })
    }

    this.gallons = function() {
        return this.getData({
            'de-de': 'gallonen',
            'en-us': 'gallons',
        })
    }

    this.gearDamage = function() {
        return this.getData({
            'de-de': 'Getriebeschaden',
            'en-us': 'Gear damage',
        })
    }

    this.cabinDamage = function() {
        return this.getData({
            'de-de': 'Kabinenschaden',
            'en-us': 'Cabin damage',
        })
    }

    this.chassisDamage = function() {
        return this.getData({
            'de-de': 'Fahrgestellschaden',
            'en-us': 'Chassis damage',
        })
    }

    this.wheelDamage = function() {
        return this.getData({
            'de-de': 'Reifenschadem',
            'en-us': 'Mature damage',
        })
    }

    this.getData = function(object) {
        return (object.hasOwnProperty(options.lang))
        ? object[options.lang]
        : (object.hasOwnProperty(options.lang.split('-')[0])
            ? object[options.lang].split('-')[0]
            : object['en-us']);
    }
}